@@ -2,9 +2,17 @@ from __future__ import annotations
 
 import uuid
 from datetime import datetime
-from typing import Any, Union
+from typing import Any, Union, Annotated
+import re
+from pydantic import AliasChoices, BaseModel, ConfigDict, Field, field_validator, BeforeValidator
+from enum import Enum
 
-from pydantic import BaseModel, ConfigDict, Field, field_validator
+
+class SenderRole(Enum):
+    USER = "user"
+    AGENT = "agent"
+    ASSISTANT = "assistant"  # Maintain backward compatibility because we have both agent and assistant
+    BOT = "bot"
 
 
 class AgentBase(BaseModel):
@@ -17,6 +25,19 @@ class AgentCreate(AgentBase):
     pass
 
 
+class Specialization(BaseModel):
+    # Decide specialization in a specific domain
+    domain: str
+    role: str
+    task: str
+
+
+class AgentUpdate(BaseModel):
+    name: str | None = None
+    description: str | None = None
+    config: dict[str, Any] | None = None
+
+
 class AgentDeployRequest(BaseModel):
     """Request schema for agent deployment endpoint"""
 
@@ -87,17 +108,36 @@ class DocumentCreate(DocumentBase):
 
 
 class DocumentRead(DocumentBase):
-    id: int
+    id: int | None = None
     filename: str
     file_size: int
     mime_type: str
     source_document_id: int | None = None
-    created_at: datetime
-    updated_at: datetime
+    created_at: datetime | None = None
+    updated_at: datetime | None = None
+    metadata: dict[str, Any] | None = Field(default_factory=dict, validation_alias=AliasChoices("doc_metadata", "metadata"))
+
+    # Additional computed metadata fields
+    file_extension: str | None = None
+    file_size_mb: float | None = None
+    is_extraction_file: bool = False
+    days_since_created: int | None = None
+    days_since_updated: int | None = None
 
     model_config = ConfigDict(from_attributes=True)
 
 
+class DocumentListResponse(BaseModel):
+    """Response schema for document list endpoint with metadata."""
+
+    documents: list[DocumentRead]
+    total: int
+    limit: int
+    offset: int
+    has_more: bool
+    metadata: dict[str, Any] = Field(default_factory=dict)
+
+
 class DocumentUpdate(BaseModel):
     original_filename: str | None = None
     topic_id: int | None = None
@@ -125,7 +165,9 @@ class RunNAFileResponse(BaseModel):
 
 class ConversationBase(BaseModel):
     title: str
-    agent_id: int
+    agent_id: int | None = None
+    kp_id: int | None = None
+    type: str | None = None
 
 
 class ConversationCreate(ConversationBase):
@@ -141,8 +183,13 @@ class ConversationRead(ConversationBase):
 
 
 class MessageBase(BaseModel):
-    sender: str
+    sender: SenderRole = Field(default=SenderRole.USER)
     content: str
+    require_user: bool = False
+    treat_as_tool: bool = False
+    metadata: dict = {}
+
+    model_config = ConfigDict(use_enum_values=True)
 
 
 class MessageCreate(MessageBase):
@@ -206,11 +253,13 @@ class ChatResponse(BaseModel):
 class MessageData(BaseModel):
     """Schema for a single message in conversation"""
 
-    role: str  # 'user' or 'assistant'
+    role: SenderRole  # 'user' or 'assistant'
     content: str
     require_user: bool = False
     treat_as_tool: bool = False
 
+    model_config = ConfigDict(use_enum_values=True)
+
 
 class AgentGenerationRequest(BaseModel):
     """Request schema for Georgia training endpoint"""
@@ -464,6 +513,11 @@ class DomainNode(BaseModel):
     topic: str
     children: list[DomainNode] = []
 
+    @property
+    def fd_name(self) -> str:
+        topic = self.topic
+        return re.sub(r"[^a-zA-Z0-9]+", "_", topic)
+
 
 class DomainKnowledgeTree(BaseModel):
     """Complete domain knowledge tree structure"""
@@ -543,6 +597,12 @@ class RevertDomainKnowledgeRequest(BaseModel):
     version_id: int
 
 
+class DeleteTopicKnowledgeRequest(BaseModel):
+    """Request to delete topic knowledge content"""
+
+    topic_parts: list[str]
+
+
 class ChatWithIntentRequest(BaseModel):
     """Extended chat request with intent detection"""
 
@@ -608,18 +668,18 @@ class ExtractionResponse(BaseModel):
 
 class WorkflowExecutionRequest(BaseModel):
     """Request schema for workflow execution endpoint"""
-    
+
     agent_id: int
     workflow_name: str
     input_data: dict[str, Any] = Field(default_factory=dict)
     execution_mode: str = "sync"  # sync, async, step-by-step
-    
+
     model_config = ConfigDict(from_attributes=True)
 
 
 class WorkflowExecutionResponse(BaseModel):
     """Response schema for workflow execution endpoint"""
-    
+
     success: bool
     execution_id: str
     status: str  # idle, running, completed, failed, paused, cancelled
@@ -629,13 +689,13 @@ class WorkflowExecutionResponse(BaseModel):
     result: Any = None
     error: str | None = None
     step_results: list[dict[str, Any]] = Field(default_factory=list)
-    
+
     model_config = ConfigDict(from_attributes=True)
 
 
 class WorkflowExecutionStatus(BaseModel):
     """Schema for workflow execution status updates"""
-    
+
     execution_id: str
     workflow_name: str
     status: str
@@ -645,26 +705,81 @@ class WorkflowExecutionStatus(BaseModel):
     step_results: list[dict[str, Any]]
     error: str | None = None
     last_update: datetime
-    
+
     model_config = ConfigDict(from_attributes=True)
 
 
 class WorkflowExecutionControl(BaseModel):
     """Schema for workflow execution control commands"""
-    
+
     execution_id: str
     action: str  # start, stop, pause, resume, cancel
-    
+
     model_config = ConfigDict(from_attributes=True)
 
 
 class WorkflowExecutionControlResponse(BaseModel):
     """Response schema for workflow execution control"""
-    
+
     success: bool
     execution_id: str
     new_status: str
     message: str
     error: str | None = None
-    
+
     model_config = ConfigDict(from_attributes=True)
+
+
+class KnowledgePackOutput(BaseModel):
+    id: int
+    folder_path: Annotated[str, BeforeValidator(lambda v: str(v))]
+    kp_metadata: dict = {}
+    created_at: datetime
+    updated_at: datetime
+
+    def get_specialization_info(self) -> Specialization:
+        return Specialization(
+            domain=self.kp_metadata.get("domain", "General"),
+            role=self.kp_metadata.get("role", "Domain Expert"),
+            task=self.kp_metadata.get("task", "Answer Questions"),
+        )
+
+
+class PaginationInfo(BaseModel):
+    """Pagination metadata for list endpoints"""
+
+    page: int
+    per_page: int
+    total: int
+    total_pages: int
+    has_next: bool
+    has_previous: bool
+    next_page: int | None
+    previous_page: int | None
+
+
+class PaginatedKnowledgePackResponse(BaseModel):
+    """Paginated response for knowledge pack listings"""
+
+    data: list[KnowledgePackOutput]
+    pagination: PaginationInfo
+
+
+class KnowledgePackCreateRequest(BaseModel):
+    kp_metadata: Specialization
+
+
+class KnowledgePackUpdateRequest(KnowledgePackCreateRequest):
+    kp_id: int
+
+
+class KnowledgePackUpdateResponse(DomainKnowledgeUpdateResponse):
+    pass
+
+
+class KnowledgePackSmartChatResponse(BaseModel):
+    success: bool
+    is_tree_modified: bool = False
+    agent_response: str
+    internal_conversation: list[MessageData] = []
+    error: str | None = None
